package sun.java2d.pipe;

import sun.java2d.SurfaceData;
import sun.java2d.loops.FillSpans;
import java.awt.BasicStroke;
import java.awt.geom.Path2D;
import java.awt.Shape;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.RoundRectangle2D;
import sun.java2d.SunGraphics2D;

public class LoopPipe implements PixelDrawPipe, PixelFillPipe, ParallelogramPipe, ShapeDrawPipe, LoopBasedPipe
{
    static final RenderingEngine RenderEngine;
    
    public LoopPipe() {
    }
    
    @Override
    public void drawLine(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        int transX = sunGraphics2D.transX;
        int transY = sunGraphics2D.transY;
        sunGraphics2D.loops.drawLineLoop.DrawLine(sunGraphics2D, sunGraphics2D.getSurfaceData(), n + transX, n2 + transY, n3 + transX, n4 + transY);
    }
    
    @Override
    public void drawRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        sunGraphics2D.loops.drawRectLoop.DrawRect(sunGraphics2D, sunGraphics2D.getSurfaceData(), n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
    }
    
    @Override
    public void drawRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        sunGraphics2D.shapepipe.draw(sunGraphics2D, new RoundRectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6));
    }
    
    @Override
    public void drawOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        sunGraphics2D.shapepipe.draw(sunGraphics2D, new Ellipse2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    public void drawArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        sunGraphics2D.shapepipe.draw(sunGraphics2D, new Arc2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6, 0));
    }
    
    @Override
    public void drawPolyline(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        sunGraphics2D.loops.drawPolygonsLoop.DrawPolygons(sunGraphics2D, sunGraphics2D.getSurfaceData(), array, array2, new int[] { n }, 1, sunGraphics2D.transX, sunGraphics2D.transY, false);
    }
    
    @Override
    public void drawPolygon(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        sunGraphics2D.loops.drawPolygonsLoop.DrawPolygons(sunGraphics2D, sunGraphics2D.getSurfaceData(), array, array2, new int[] { n }, 1, sunGraphics2D.transX, sunGraphics2D.transY, true);
    }
    
    @Override
    public void fillRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        sunGraphics2D.loops.fillRectLoop.FillRect(sunGraphics2D, sunGraphics2D.getSurfaceData(), n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
    }
    
    @Override
    public void fillRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        sunGraphics2D.shapepipe.fill(sunGraphics2D, new RoundRectangle2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6));
    }
    
    @Override
    public void fillOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        sunGraphics2D.shapepipe.fill(sunGraphics2D, new Ellipse2D.Float((float)n, (float)n2, (float)n3, (float)n4));
    }
    
    @Override
    public void fillArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        sunGraphics2D.shapepipe.fill(sunGraphics2D, new Arc2D.Float((float)n, (float)n2, (float)n3, (float)n4, (float)n5, (float)n6, 2));
    }
    
    @Override
    public void fillPolygon(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        ShapeSpanIterator fillSSI = getFillSSI(sunGraphics2D);
        try {
            fillSSI.setOutputArea(sunGraphics2D.getCompClip());
            fillSSI.appendPoly(array, array2, n, sunGraphics2D.transX, sunGraphics2D.transY);
            fillSpans(sunGraphics2D, fillSSI);
        }
        finally {
            fillSSI.dispose();
        }
    }
    
    @Override
    public void draw(SunGraphics2D sunGraphics2D, Shape shape) {
        if (sunGraphics2D.strokeState == 0) {
            Path2D.Float float1;
            int transX;
            int transY;
            if (sunGraphics2D.transformState <= 1) {
                if ((shape instanceof Path2D.Float)) {
                    float1 = (Path2D.Float)shape;
                }
                else {
                    float1 = new Path2D.Float(shape);
                }
                transX = sunGraphics2D.transX;
                transY = sunGraphics2D.transY;
            }
            else {
                float1 = new Path2D.Float(shape, sunGraphics2D.transform);
                transX = 0;
                transY = 0;
            }
            sunGraphics2D.loops.drawPathLoop.DrawPath(sunGraphics2D, sunGraphics2D.getSurfaceData(), transX, transY, float1);
            return;
        }
        else {
            if (sunGraphics2D.strokeState == 3) {
                this.fill(sunGraphics2D, sunGraphics2D.stroke.createStrokedShape(shape));
                return;
            }
            ShapeSpanIterator strokeSpans = getStrokeSpans(sunGraphics2D, shape);
            try {
                fillSpans(sunGraphics2D, strokeSpans);
            }
            finally {
                strokeSpans.dispose();
            }
            return;
        }
    }
    
    public static ShapeSpanIterator getFillSSI(SunGraphics2D sunGraphics2D) {
        return new ShapeSpanIterator(sunGraphics2D.stroke instanceof BasicStroke && sunGraphics2D.strokeHint != 2);
    }
    
    public static ShapeSpanIterator getStrokeSpans(SunGraphics2D sunGraphics2D, Shape shape) {
        ShapeSpanIterator shapeSpanIterator = new ShapeSpanIterator(false);
        try {
            shapeSpanIterator.setOutputArea(sunGraphics2D.getCompClip());
            shapeSpanIterator.setRule(1);
            LoopPipe.RenderEngine.strokeTo(shape, sunGraphics2D.transform, (BasicStroke)sunGraphics2D.stroke, sunGraphics2D.strokeState <= 1, sunGraphics2D.strokeHint != 2, false, shapeSpanIterator);
        }
        catch (Throwable cause) {
            shapeSpanIterator.dispose();
            throw new InternalError("Unable to Stroke shape (" + cause.getMessage() + ")", cause);
        }
        return shapeSpanIterator;
    }
    
    @Override
    public void fill(SunGraphics2D sunGraphics2D, Shape shape) {
        if (sunGraphics2D.strokeState == 0) {
            Path2D.Float float1;
            int transX;
            int transY;
            if (sunGraphics2D.transformState <= 1) {
                if ((shape instanceof Path2D.Float)) {
                    float1 = (Path2D.Float)shape;
                }
                else {
                    float1 = new Path2D.Float(shape);
                }
                transX = sunGraphics2D.transX;
                transY = sunGraphics2D.transY;
            }
            else {
                float1 = new Path2D.Float(shape, sunGraphics2D.transform);
                transX = 0;
                transY = 0;
            }
            sunGraphics2D.loops.fillPathLoop.FillPath(sunGraphics2D, sunGraphics2D.getSurfaceData(), transX, transY, float1);
            return;
        }
        else {
            ShapeSpanIterator fillSSI = getFillSSI(sunGraphics2D);
            try {
                fillSSI.setOutputArea(sunGraphics2D.getCompClip());
                fillSSI.appendPath(shape.getPathIterator((sunGraphics2D.transformState == 0) ? null : sunGraphics2D.transform));
                fillSpans(sunGraphics2D, fillSSI);
            }
            finally {
                fillSSI.dispose();
            }
            return;
        }
    }
    
    private static void fillSpans(SunGraphics2D sg2d, SpanIterator filter) {
        if (sg2d.clipState == 2) {
            filter = sg2d.clipRegion.filter(filter);
        }
        else {
            FillSpans fillSpansLoop = sg2d.loops.fillSpansLoop;
            if (fillSpansLoop != null) {
                fillSpansLoop.FillSpans(sg2d, sg2d.getSurfaceData(), filter);
                return;
            }
        }
        int[] array = new int[4];
        SurfaceData surfaceData = sg2d.getSurfaceData();
        while (filter.nextSpan(array)) {
            int n = array[0];
            int n2 = array[1];
            sg2d.loops.fillRectLoop.FillRect(sg2d, surfaceData, n, n2, array[2] - n, array[3] - n2);
        }
    }
    
    @Override
    public void fillParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10) {
        sunGraphics2D.loops.fillParallelogramLoop.FillParallelogram(sunGraphics2D, sunGraphics2D.getSurfaceData(), n5, n6, n7, n8, n9, n10);
    }
    
    @Override
    public void drawParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12) {
        sunGraphics2D.loops.drawParallelogramLoop.DrawParallelogram(sunGraphics2D, sunGraphics2D.getSurfaceData(), n5, n6, n7, n8, n9, n10, n11, n12);
    }
    
    static {
        RenderEngine = RenderingEngine.getInstance();
    }
}
